﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Mail;
using System.Windows.Forms;
using IssueMailer.Settings;
using Styx;
using Styx.Common;
using Styx.Plugins;
using Styx.WoWInternals;
using System.Threading;
using Styx.CommonBot;

namespace IssueMailer
{
    internal class Main : HBPlugin
    {
        private Form _gui;
        public override string Name { get { return "IssueMailer"; } }
        public override string Author { get { return "Paxxx"; } }
        public override Version Version { get { return new Version(1, 4, 1, 0); } }
        public override string ButtonText { get { return "Settings"; } }
        public override bool WantButton { get { return true; } }
        private static IMSettings Settings { get { return IMSettings.Instance; } }
        private DateTime _timePassedSinceQuestDelivered;
        private bool _sentMailQuest = false;
        private bool _sentMailStuck = false;
        private uint _qc;
        private uint _qcold = 0;
        private string _name;
        private bool init = false;
        private uint _deathCount = 0;
        public bool stopMailingByAutomaticRestart = false;

        private WoWPoint _lastLocation;
        private DateTime _timePassedInSamePos;
        private DateTime _timePassedAlive;

        public override void OnButtonPress()
        {
            if (_gui == null || _gui.IsDisposed || _gui.Disposing) _gui = new GUI();
            if (_gui != null || _gui.IsDisposed) _gui.ShowDialog();
        }

        public void Initialize()
        {
            Logging.Write("[IssueMailer] v" + Version + " initialized...");
        }

        private void SendMail(string s, string b)
        {
            _name = StyxWoW.Me.Name;

            var fromAddress = new MailAddress(Settings.MailFrom, "[IssueMailer] Attention");
            var toAddress = new MailAddress(Settings.MailTo, "IssueMailer");
            string fromPassword = Settings.MailPwd;
            if (s == null) throw new ArgumentNullException(nameof(s));
            string subject = s;
            if (b == null) throw new ArgumentNullException(nameof(b));
            string body = b;

            var smtp = new SmtpClient
            {
                Host = "smtp.gmail.com",
                Port = 587,
                EnableSsl = true,
                DeliveryMethod = SmtpDeliveryMethod.Network,
                UseDefaultCredentials = false,
                Credentials = new NetworkCredential(fromAddress.Address, fromPassword)
            };

            using (var message = new MailMessage(fromAddress, toAddress)
            {
                Subject = subject,
                Body = body
            })
            {
                smtp.Send(message);
            }
        }

        public Main()
        {
            Initialize();
        }

        public override void Pulse()
        {

            if (!StyxWoW.IsInWorld || !StyxWoW.Me.IsAlive)
            {
                return;
            }

            // QUEST CHECK PART
            _qc = StyxWoW.Me.QuestLog.QuestCount;
            if (_qc != _qcold && Settings.EnableQuestCheck == true)
            {

                _sentMailQuest = false;
                _qcold = _qc;
                _timePassedSinceQuestDelivered = DateTime.Now + new TimeSpan(0, 0, Settings.MailTimeQuest, 0, 0);
                return;
            }

            if (DateTime.Now >= _timePassedSinceQuestDelivered && _sentMailQuest == false && Settings.EnableMailing == true && Settings.EnableQuestCheck == true)
            {
                //XX minutes passed
                Logging.Write("[IssueMailer]: ATTENTION: Since " + Settings.MailTimeQuest + " Minutes NO QUEST DELIVERED! Sending Mail...");

                SendMail("[IssueMailer]" + _name + " HAS NO QUEST DELIVERED SINCE " + Settings.MailTimeQuest + " MINUTES",
                        "Something seems wrong as " + _name + " delivered the last quest more than " + Settings.MailTimeQuest + " minutes ago");
                _sentMailQuest = true;
            }

            // STUCK CHECK PART
            if (!StyxWoW.Me.Location.Equals(_lastLocation) && Settings.EnableStuckCheck == true)
            {
                _sentMailStuck = false;
                _lastLocation = StyxWoW.Me.Location;
                _timePassedInSamePos = DateTime.Now + new TimeSpan(0, 0, Settings.MailTimeStuck, 0, 0);
                return;
            }

            if (DateTime.Now >= _timePassedInSamePos && _sentMailStuck == false && Settings.EnableMailing == true && Settings.EnableStuckCheck == true && !StyxWoW.Me.IsActuallyInCombat)
            {
                //XX minutes passed 
                Logging.Write("[IssueMailer]: ATTENTION: Not moved since " + Settings.MailTimeStuck + " Minutes - STUCK ALERT! Sending Mail...");
                SendMail("[IssueMailer]" + _name + " STUCKS since " + Settings.MailTimeStuck + " MINUTES",
                        "Something seems wrong as " + _name + " is not moving for more than " + Settings.MailTimeStuck + " minutes");
                Logging.Write("[IssueMailer]: Trying now stopping and restarting bot.");

                stopMailingByAutomaticRestart = true;
                restart();
                _sentMailStuck = true;
            }

        }

        public override void OnEnable()
        {

            if (!init)
            {
                init = true;
                if (Settings.EnableWhispers)
                {
                    Lua.Events.AttachEvent("CHAT_MSG_WHISPER", handleWhispers);
                    Lua.Events.AttachEvent("CHAT_MSG_BN_WHISPER", handleWhispersBN);
                }
                if (Settings.EnableDeathMail)
                {
                    Lua.Events.AttachEvent("PLAYER_DEAD", handleDeath);
                }
                if (Settings.EnableStopNotifier)
                {
                    Styx.CommonBot.BotEvents.OnBotStopped += BotEvents_OnBotStopped;
                }
            }
        }



        public override void OnDisable()
        {
            if (Settings.EnableWhispers)
            {
                Lua.Events.DetachEvent("CHAT_MSG_WHISPER", handleWhispers);
                Lua.Events.DetachEvent("CHAT_MSG_BN_WHISPER", handleWhispersBN);
            }
            if (Settings.EnableDeathMail)
            {
                Lua.Events.DetachEvent("PLAYER_DEAD", handleDeath);
            }
            if (Settings.EnableStopNotifier)
            {
                Styx.CommonBot.BotEvents.OnBotStopped -= BotEvents_OnBotStopped;

            }
            init = false;
        }

        private void BotEvents_OnBotStopped(EventArgs args)
        {
            if (!stopMailingByAutomaticRestart) SendMail(StyxWoW.Me.Name + " - Bot stopped", "Attention! Honorbuddy stopped.");
        }

        private void handleWhispersBN(object sender, LuaEventArgs args)
        {
            whispers(args.Args[0].ToString(), "BN");
        }

        private void handleWhispers(object sender, LuaEventArgs args)
        {
            string author = args.Args[1].ToString();
            if (author == StyxWoW.Me.Name)
                author = "Me";
            whispers(args.Args[0].ToString(), author);

        }

        private void whispers(string msg, string author)
        {
            Logging.Write(string.Format("[Issue Mailer] ATTENTION: Received whisper from: {0}", author));
            Logging.Write(string.Format("[Issue Mailer] Message: {0}", msg));
            SendMail(_name + "got a whisper from:" + author, "Message: " + msg);
        }

        private void handleDeath(object sender, LuaEventArgs args)
        {
            if (_deathCount == 0)
            {
                Logging.Write("[Issue Mailer] INFO: Death Countdown initialized...");
                _timePassedAlive = DateTime.Now + new TimeSpan(0, 0, Settings.MailDeathTime, 0, 0);
            }

            _deathCount++;
            if (_deathCount >= 1 && DateTime.Now >= _timePassedAlive)
            {
                Logging.Write("[Issue Mailer] INFO: Timeframe for deaths timed out - reseting death count...");
                _deathCount = 0;
            }

            if (_deathCount >= Settings.MailDeathCount)
            {
                Logging.Write("[Issue Mailer] ATTENTION: Too many deaths within timeframe - sending out mail...");
                this.death();
                _deathCount = 0;
            }
        }

        private void death()
        {
            SendMail(_name + " died " + _deathCount + " times now within " + Settings.MailDeathTime + " minutes!", "Keep Attention! Maybe something is wrong because you die a lot!");
        }

        private void restart()
        {
            ThreadPool.QueueUserWorkItem(o =>
            {
                TreeRoot.Stop();
                TreeRoot.WaitForStop();
                TreeRoot.Start();
                stopMailingByAutomaticRestart = false;
            });


        }
    }
}